from PyQt4.QtCore import QObject, pyqtSignal

from .slicemodel import SliceDirection
import numpy as np
import segyio
import os


class EmptyDataSource(object):
    def __init__(self):
        super(EmptyDataSource, self).__init__()
        self._data = np.zeros((2, 2), dtype=np.single)
        self._data[0, 0] = -1.0
        self._data[1, 1] = 1.0

    @property
    def ilines(self):
        return [0, 1]

    @property
    def xlines(self):
        return [0, 1]

    @property
    def samples(self):
        return [0, 1]

    @property
    def iline(self):
        return [self._data, self._data]

    @property
    def xline(self):
        return [self._data, self._data]

    @property
    def iline(self):
        return [self._data, self._data]

    @property
    def depth_slice(self):
        return [self._data, self._data]

    @property
    def sorting(self):
        return segyio.TraceSortingFormat.CROSSLINE_SORTING


class SliceDataSource(QObject):
    slice_data_source_changed = pyqtSignal()

    def __init__(self, filename, **kwargs):
        QObject.__init__(self)

        self._file_size = 0
        self._source = None
        """ :type: segyio.SegyFile """
        self.set_source_filename(filename, **kwargs)

    def _close_current_file(self):
        if isinstance(self._source, segyio.SegyFile):
            self._source.close()

        self._file_size = 0
        self._source = None

    @property
    def file_size(self):
        return self._file_size

    def set_source_filename(self, filename, **kwargs):
        if filename:
            try:
                file_size = os.stat(filename).st_size
                source = segyio.open(filename, "r", **kwargs)
            except:
                raise
            else:
                self._close_current_file()
                self._source = source
                self._file_size = file_size
                self._source.mmap()
                samples = self._source.samples
        else:
            self._close_current_file()
            self._source = EmptyDataSource()

        self.slice_data_source_changed.emit()

    def read_slice(self, direction, index):
        if direction == SliceDirection.inline:
            iline_index = self._source.ilines[index]
            return self._source.iline[iline_index].T
        elif direction == SliceDirection.crossline:
            xline_index = self._source.xlines[index]
            return self._source.xline[xline_index].T
        elif direction == SliceDirection.depth:
            data = self._source.depth_slice[index]
            if self._source.sorting == segyio.TraceSortingFormat.CROSSLINE_SORTING:
                return data
            else:
                return data.T
        else:
            raise ValueError("Unknown direction: %s" % direction)

    def indexes_for_direction(self, direction):
        if direction == SliceDirection.inline:
            return self._source.ilines
        elif direction == SliceDirection.crossline:
            return self._source.xlines
        elif direction == SliceDirection.depth:
            return self._source.samples
        else:
            raise ValueError("Unknown direction: %s" % direction)

    def __del__(self):
        self._close_current_file()

    def dims(self):
        iline_count = len(self._source.ilines)
        xline_count = len(self._source.xlines)
        offset_count = 1
        sample_count = len(self._source.samples)
        return iline_count, xline_count, offset_count, sample_count



from __future__ import division
from PyQt4.QtGui import QCheckBox, QWidget, QComboBox, QDoubleSpinBox, QLabel
from PyQt4.QtGui import QHBoxLayout


class PlotExportSettingsWidget(QWidget):
    def __init__(self, parent, slice_view_widget, context):
        super(PlotExportSettingsWidget, self).__init__(parent)

        self._slice_view_widget = slice_view_widget
        self._context = context

        self._dpi_units = ["in", "cm", "px"]

        if parent is None:
            w, h, dpi = 11.7, 8.3, 100
        else:
            fig = self._slice_view_widget.layout_figure()
            w, h = fig.get_size_inches()
            dpi = fig.dpi

        self._label = QLabel()
        self._label.setDisabled(True)
        self._set_label_txt(w, h, self._dpi_units[0])

        self._fix_size = QCheckBox()
        self._fix_width = QDoubleSpinBox()
        self._fix_width.setDisabled(True)

        self._fix_height = QDoubleSpinBox()
        self._fix_height.setDisabled(True)

        self._fix_dpi_units = QComboBox()
        self._fix_dpi_units.setDisabled(True)

        self._fix_width.setMinimum(1)
        self._fix_width.setMaximum(32000)
        self._fix_width.setValue(w)

        self._fix_height.setMinimum(1)
        self._fix_height.setMaximum(32000)
        self._fix_height.setValue(h)

        self._fix_width.valueChanged.connect(self._fixed_image)
        self._fix_height.valueChanged.connect(self._fixed_image)

        self._fix_dpi_units.addItems(self._dpi_units)
        self._fix_dpi_units.activated.connect(self._fixed_image)
        self._fix_size.toggled.connect(self._fixed_image)

        self._label_widget = self._label
        self._enable_widget = self._fix_size
        self._dpi_widget = self._fix_dpi_units
        self._height_widget = self._fix_height
        self._width_widget = self._fix_width

    @property
    def label(self):
        return self._label_widget

    @property
    def checkbox(self):
        return self._enable_widget

    @property
    def width_spinbox(self):
        return self._width_widget

    @property
    def height_spinbox(self):
        return self._height_widget

    @property
    def units_combobox(self):
        return self._dpi_widget

    @staticmethod
    def to_inches(width, height, dpi, scale):
        if scale == "in":
            return (width, height, dpi)
        elif scale == "cm":
            return (width / 2.54, height / 2.54, dpi)
        elif scale == "px":
            return (width / dpi, height / dpi, dpi)
        else:
            raise NotImplementedError

    def _set_label_txt(self, w, h, scale):
        self._label.setText("{0} x {1} {2}".format(w, h, scale))

    def _fixed_image(self):
        ctx = self._context

        # toggle disabled
        self._fix_height.setDisabled(not self._fix_size.isChecked())
        self._fix_width.setDisabled(not self._fix_size.isChecked())
        self._fix_dpi_units.setDisabled(not self._fix_size.isChecked())
        self._label.setDisabled(not self._fix_size.isChecked())

        if not self._fix_size.isChecked():
            ctx.set_image_size(None)
            return

        w = self._fix_width.value()
        h = self._fix_height.value()
        dpi = 100
        scale = self._fix_dpi_units.currentText()

        self._set_label_txt(w, h, scale)
        ctx.set_image_size(*self.to_inches(w, h, dpi, scale))



from unittest import TestCase

from segyviewlib import SliceModel, SliceDirection as SD
import numpy as np


class SliceModelTest(TestCase):
    def test_default_model(self):
        model = SliceModel("test", SD.inline, SD.crossline, SD.depth)

        self.assertEqual(model.title, "test")
        self.assertEqual(model.x_axis_name, SD.crossline['name'])
        self.assertEqual(model.y_axis_name, SD.depth['name'])
        self.assertIsNone(model.data)
        self.assertEqual(model.index, 0)
        self.assertIsNone(model.max_value)
        self.assertEqual(model.x_index, 0)
        self.assertEqual(model.y_index, 0)
        self.assertIsNone(model.indexes)
        self.assertIsNone(model.x_indexes)
        self.assertIsNone(model.y_indexes)
        self.assertTrue(model.visible)
        self.assertFalse(model.dirty)

        self.assertEqual(model.index_direction, SD.inline)
        self.assertEqual(model.x_index_direction, SD.crossline)
        self.assertEqual(model.y_index_direction, SD.depth)

        with self.assertRaises(TypeError):
            r = model.min_x

        with self.assertRaises(TypeError):
            r = model.min_y

        with self.assertRaises(TypeError):
            r = model.max_x

        with self.assertRaises(TypeError):
            r = model.max_y

        with self.assertRaises(TypeError):
            r = model.width

        with self.assertRaises(TypeError):
            r = model.height

    def test_minimal_model_with_data(self):
        model = SliceModel("test", SD.inline, SD.crossline, SD.depth)
        data = np.zeros((3, 5))
        data[0, 0] = -1.5
        data[2, 3] = 1.5
        model.data = data

        self.assertIsInstance(model.data, np.ndarray)
        self.assertListEqual(model.x_indexes, [0, 1, 2])
        self.assertListEqual(model.y_indexes, [0, 1, 2, 3, 4])
        self.assertEqual(model.min_x, 0)
        self.assertEqual(model.min_y, 0)
        self.assertEqual(model.max_x, 2)
        self.assertEqual(model.max_y, 4)
        self.assertAlmostEqual(model.min_value, -1.5)
        self.assertAlmostEqual(model.max_value, 1.5)
        self.assertEqual(model.width, 3)
        self.assertEqual(model.height, 5)

    def test_adjusted_model_with_data(self):
        model = SliceModel("test", SD.inline, SD.crossline, SD.depth)
        model.data = np.zeros((3, 5))

        model.x_indexes = [2, 4, 6, 8, 10]
        model.y_indexes = [1, 3, 5]
        self.assertListEqual(model.x_indexes, [2, 4, 6, 8, 10])
        self.assertListEqual(model.y_indexes, [1, 3, 5])
        self.assertEqual(model.min_x, 2)
        self.assertEqual(model.min_y, 1)
        self.assertEqual(model.max_x, 10)
        self.assertEqual(model.max_y, 5)
        self.assertEqual(model.width, 5)
        self.assertEqual(model.height, 3)

        with self.assertRaises(ValueError):
            model.x_indexes = [2, 4, 6, 8]

        with self.assertRaises(ValueError):
            model.y_indexes = [2, 4, 6, 8]

        with self.assertRaises(ValueError):
            model.data = np.zeros((2, 2))

    def test_formatters(self):
        model = SliceModel("test", SD.inline, SD.crossline, SD.depth)
        model.x_indexes = [1, 3, 7, 9]
        model.y_indexes = [2, 4, 6, 8]

        self.assertEqual(model.x_axis_formatter(0, 0), 1)
        self.assertEqual(model.x_axis_formatter(2, 0), 7)
        self.assertEqual(model.x_axis_formatter(-1, 0), '')

        self.assertEqual(model.y_axis_formatter(0, 0), 2)
        self.assertEqual(model.y_axis_formatter(2, 0), 6)
        self.assertEqual(model.y_axis_formatter(7, 0), '')

    def test_simple_setters(self):
        model = SliceModel("test", SD.inline, SD.crossline, SD.depth)
        model.indexes = [1, 2, 3]
        model.index = 5
        model.x_index = 99
        model.y_index = 33
        model.visible = False

        self.assertTrue(model.dirty)
        model.data = np.zeros((2, 2))
        self.assertFalse(model.dirty)

        self.assertEqual(model.index, 5)
        self.assertEqual(model.indexes, [1, 2, 3])
        self.assertEqual(len(model), 3)
        self.assertEqual(model.x_index, 99)
        self.assertEqual(model.y_index, 33)
        self.assertFalse(model.visible)

    def test_reset(self):
        model = SliceModel("test", SD.inline, SD.crossline, SD.depth)
        model.data = np.zeros((3, 5))
        model.data[0, 0] = -1.0
        model.data[1, 0] = 1.0

        model.x_indexes = [2, 4, 6, 8, 10]
        model.y_indexes = [1, 3, 5]

        model.indexes = [1, 2, 3]
        model.index = 5
        model.x_index = 99
        model.y_index = 33

        model.reset()

        self.assertIsNone(model.data)
        self.assertEqual(model.index, 0)
        self.assertIsNone(model.max_value)
        self.assertEqual(model.x_index, 0)
        self.assertEqual(model.y_index, 0)
        self.assertIsNone(model.indexes)
        self.assertIsNone(model.x_indexes)
        self.assertIsNone(model.y_indexes)


from PyQt4.QtCore import QPoint
from PyQt4.QtGui import QMenu, QAction
from math import copysign

from segyviewlib import SliceView, LayoutCanvas, SliceModel
from matplotlib.cm import ScalarMappable


class SliceViewWidget(LayoutCanvas):
    def __init__(self, context, width=11.7, height=8.3, dpi=100, parent=None):
        """ :type context: segyviewlib.SliceViewContext """
        super(SliceViewWidget, self).__init__(width, height, dpi, parent)

        self._assigned_slice_models = list(context.models)
        """ :type: list[SliceModel] """
        self._slice_views = {}
        """ :type: dict[matplotlib.axes.Axes,SliceView] """

        self._colormappable = ScalarMappable(cmap=context.colormap)
        self._colormappable.set_array([])

        self._context = context
        context.context_changed.connect(self._context_changed)
        context.data_changed.connect(self._data_changed)
        context.data_source_changed.connect(self._layout_changed)

        self.layout_changed.connect(self._layout_changed)
        self.subplot_pressed.connect(self._subplot_clicked)
        self.subplot_scrolled.connect(self._subplot_scrolled)
        self.subplot_motion.connect(self._subplot_motion)

    def _create_context(self):
        return self._context.create_context(self._assigned_slice_models)

    def _layout_changed(self):
        fig = self.layout_figure()
        axes = fig.layout_axes()
        self._slice_views.clear()

        for model in self._context.models:
            model.visible = False

        for index, ax in enumerate(axes):
            ax.clear()
            if index < len(self._assigned_slice_models):
                model = self._assigned_slice_models[index]
                model.visible = True
                self._slice_views[ax] = SliceView(ax, model)
                self._slice_views[ax].create_slice(self._create_context())

        colormap_axes = self.layout_figure().colormap_axes()
        colorbar = self.layout_figure().colorbar(self._colormappable, cax=colormap_axes, use_gridspec=True)
        colorbar.ax.tick_params(labelsize=9)

        self._context.load_data()
        self._data_changed()

    def _data_changed(self):
        context = self._create_context()
        for slice_view in self._slice_views.values():
            slice_view.data_changed(context)
        self._context_changed()

    def _context_changed(self):
        ctx = self._create_context()
        for slice_view in self._slice_views.values():
            slice_view.context_changed(ctx)

        self._colormappable.set_cmap(ctx['colormap'])
        self._colormappable.set_clim(ctx['min'], ctx['max'])
        self.draw()

    def _create_slice_view_context_menu(self, subplot_index):
        context_menu = QMenu("Slice Model Reassignment Menu", self)

        def reassign(model):
            def fn():
                self._assigned_slice_models[subplot_index] = model
                self._layout_changed()

            return fn

        for model in self._context.models:
            action = QAction(model.title, self)
            action.triggered.connect(reassign(model))
            context_menu.addAction(action)

        return context_menu

    def _subplot_clicked(self, event):
        keys = event['key']

        if self._context.indicators and event['button'] == 1 and not keys:
            x = int(event['x'])
            y = int(event['y'])
            slice_model = self._get_slice_view(event).model()
            self._context.update_index_for_direction(slice_model.x_index_direction, x)
            self._context.update_index_for_direction(slice_model.y_index_direction, y)

        elif event['button'] == 3 and (not keys or keys.state(ctrl=True)):
            subplot_index = event['subplot_index']
            context_menu = self._create_slice_view_context_menu(subplot_index)
            context_menu.exec_(self.mapToGlobal(QPoint(event['mx'], self.height() - event['my'])))

    def _subplot_scrolled(self, event):
        keys = event['key']
        if not keys:
            # at least 1 step per event but a maximum of 5
            step = max(1, abs(int(event['step'])))
            step = min(step, 5)
            step = copysign(step, event['step'])

            slice_model = self._get_slice_view(event).model()
            index = int(slice_model.index + step)

            if 0 <= index < len(slice_model):
                self._context.update_index_for_direction(slice_model.index_direction, index)

        elif keys.state(ctrl=True) or keys.state(shift=True):
            x = event['x']
            y = event['y']
            step = max(1, abs(int(event['step'])))
            step = copysign(step / 20.0, event['step'])

            slice_view = self._get_slice_view(event)

            if slice_view.zoom(x, y, step):
                self._context_changed()

    def _get_slice_view(self, event):
        subplot_index = event['subplot_index']
        fig = self.layout_figure()
        ax = fig.layout_axes()[subplot_index]
        slice_view = self._slice_views[ax]
        return slice_view

    def _subplot_motion(self, event):
        keys = event['key']
        if keys.state(shift=True) or keys.state(ctrl=True):
            dx = event['dx']
            dy = event['dy']

            if dx is not None and dy is not None:
                slice_view = self._get_slice_view(event)
                slice_view.pan(dx, dy)
                self._context_changed()



#!/usr/bin/env python
import sys

import matplotlib.cm as mcm
from PyQt4.QtCore import Qt
from PyQt4.QtGui import QMainWindow, QWidget, QApplication, QLabel, QVBoxLayout

from segyviewlib import LayoutCombo, ColormapCombo, LayoutCanvas


class TestGUI(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)

        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setWindowTitle("GUI Test")

        toolbar = self.addToolBar("Stuff")
        """:type: QToolBar"""

        layout_combo = LayoutCombo()
        toolbar.addWidget(layout_combo)
        layout_combo.layout_changed.connect(self._layout_changed)

        self._colormap_combo = ColormapCombo()
        toolbar.addWidget(self._colormap_combo)
        self._colormap_combo.currentIndexChanged[int].connect(self._colormap_changed)

        central_widget = QWidget()
        layout = QVBoxLayout()
        central_widget.setLayout(layout)

        self._layout_canvas = LayoutCanvas(width=5, height=5)
        self._layout_canvas.set_plot_layout(layout_combo.get_current_layout())
        layout.addWidget(self._layout_canvas)

        colormap_name = str(self._colormap_combo.itemText(0))
        self._colormap = mcm.ScalarMappable(cmap=colormap_name)
        self._colormap.set_array([])

        self._colormap_label = QLabel()
        layout.addWidget(self._colormap_label)

        self.setCentralWidget(central_widget)

        self._update_axes()

    def _update_axes(self):
        layout_figure = self._layout_canvas.layout_figure()
        colormap_axes = layout_figure.colormap_axes()
        self._colorbar = layout_figure.colorbar(self._colormap, cax=colormap_axes, use_gridspec=True)
        self._colormap_changed(self._colormap_combo.currentIndex())

    def _layout_changed(self, layout):
        self._layout_canvas.set_plot_layout(layout)
        self._update_axes()

    def _colormap_changed(self, index):
        colormap_name = str(self._colormap_combo.itemText(index))
        self._colormap.set_cmap(colormap_name)
        self._colormap_label.setText("Colormap selected: %s" % colormap_name)
        self._layout_canvas.draw()


if __name__ == '__main__':
    if len(sys.argv) > 1:
        sys.exit()
    else:
        q_app = QApplication(sys.argv)

        gui = TestGUI()
        gui.show()
        gui.raise_()
        sys.exit(q_app.exec_())



from PyQt4.QtGui import QSpinBox, QValidator


class ArraySpinBox(QSpinBox):
    def __init__(self, values, parent=None):
        QSpinBox.__init__(self, parent)
        self.setKeyboardTracking(False)
        self._values = []
        """ :type: list[int]"""
        self.set_index_values(values)
        self.setMinimum(0)

    def update_view(self, indexes, index):
        self.blockSignals(True)
        self.set_index_values(indexes)
        self.setValue(index)
        self.blockSignals(False)

    def set_index_values(self, values):
        self._values = values
        if self.maximum() != len(values) - 1:
            self.setMaximum(len(values) - 1)

    def setValue(self, value):
        QSpinBox.setValue(self, value)

    def valueFromText(self, text):
        text = str(text)
        if text.strip() == "":
            index = 0
        else:
            value = int(text)
            index = self._values.index(value)
        return index

    def textFromValue(self, index):
        val = self._values[index]
        if isinstance(val, float):
            val = round(val, 4)
        return str(val)

    def validate(self, text, pos):
        text = str(text)
        if text.strip() == "":
            return QValidator.Acceptable, pos

        try:
            value = int(text)
        except ValueError:
            return QValidator.Invalid, pos

        try:
            index = self._values.index(value)
        except ValueError:
            for value in self._values:
                if str(value).startswith(text[:pos]):
                    return QValidator.Intermediate, pos
            return QValidator.Invalid, pos

        return QValidator.Acceptable, pos



import sys
from PyQt4.QtGui import QCheckBox, QWidget
from PyQt4.QtGui import QHBoxLayout, QDoubleSpinBox
from PyQt4.QtCore import pyqtSignal, QObject


class SampleScaleController(QObject):
    min_max_changed = pyqtSignal(tuple)

    def __init__(self, parent=None):
        QObject.__init__(self, parent)

        self._min_active, self._min_spinbox = self._set_up_opt_spin_box()

        self._min_active.toggled.connect(self._value_changed)
        self._min_spinbox.valueChanged.connect(self._value_changed)

        self._max_active, self._max_spinbox = self._set_up_opt_spin_box()

        self._max_active.toggled.connect(self._value_changed)
        self._max_spinbox.valueChanged.connect(self._value_changed)

    def _set_up_opt_spin_box(self):
        check_box = QCheckBox()
        spin_box = QDoubleSpinBox()
        spin_box.setDecimals(5)
        spin_box.setSingleStep(0.01)
        spin_box.setMinimum(-sys.float_info.max)
        spin_box.setMaximum(sys.float_info.max)
        spin_box.setDisabled(True)
        check_box.toggled.connect(spin_box.setEnabled)

        return check_box, spin_box

    def _value_changed(self):
        min_value = None
        max_value = None
        if self._min_active.isChecked():
            min_value = self._min_spinbox.value()
            self._max_spinbox.setMinimum(min_value)
        else:
            self._max_spinbox.setMinimum(-sys.float_info.max)

        if self._max_active.isChecked():
            max_value = self._max_spinbox.value()
            self._min_spinbox.setMaximum(max_value)
        else:
            self._min_spinbox.setMaximum(sys.float_info.max)

        self.min_max_changed.emit((min_value, max_value))

    @property
    def min_checkbox(self):
        return self._min_active

    @property
    def min_spinbox(self):
        return self._min_spinbox

    @property
    def max_checkbox(self):
        return self._max_active

    @property
    def max_spinbox(self):
        return self._max_spinbox



from PyQt4.QtCore import QObject
from PyQt4.QtCore import pyqtSignal

from segyviewlib import SliceModel, SliceDataSource, SliceDirection


class ViewLimit(object):
    def __init__(self, model):
        self._min_xlim = 0
        self._max_xlim = model.width
        self._min_ylim = 0
        self._max_ylim = model.height

    @property
    def max_xlim(self):
        return self._max_xlim

    @max_xlim.setter
    def max_xlim(self, value):
        self._max_xlim = value + 1  # setting the constraint to the index above to the max

    @property
    def min_xlim(self):
        return self._min_xlim

    @min_xlim.setter
    def min_xlim(self, value):
        self._min_xlim = value

    @property
    def min_ylim(self):
        return self._min_ylim

    @min_ylim.setter
    def min_ylim(self, value):
        self._min_ylim = value

    @property
    def max_ylim(self):
        return self._max_ylim

    @max_ylim.setter
    def max_ylim(self, value):
        self._max_ylim = value + 1  # setting the constraint to the index above to the max


class SliceViewContext(QObject):
    context_changed = pyqtSignal()
    data_changed = pyqtSignal()
    data_source_changed = pyqtSignal()

    def __init__(self, slice_models, slice_data_source, colormap='seismic', interpolation='nearest', image_size=None):
        QObject.__init__(self)

        self._available_slice_models = slice_models
        """ :type: list[SliceModel] """
        self._slice_data_source = slice_data_source
        """ :type: SliceDataSource """

        self._slice_data_source.slice_data_source_changed.connect(self._reset)

        self._colormap_name = colormap
        self._show_indicators = False
        self._global_min = None
        self._global_max = None
        self._user_min_value = None
        self._user_max_value = None
        self._samples_unit = "Time (ms)"

        self._interpolation = interpolation
        self._symmetric_scale = True
        self._image_size = image_size

        self._assign_indexes()

        self._view_limits = {}
        for model in self._available_slice_models:
            self._view_limits[model.index_direction['name']] = ViewLimit(model)

    @property
    def models(self):
        """ :rtype: list[SliceModel]"""
        return self._available_slice_models

    @property
    def colormap(self):
        """ :rtype: str """
        return self._colormap_name

    @property
    def indicators(self):
        """ :rtype: bool """
        return self._show_indicators

    @property
    def interpolation(self):
        """ :rtype: str """
        return self._interpolation

    @property
    def global_minimum(self):
        """ :rtype: float """
        return self._global_min

    @property
    def global_maximum(self):
        """ :rtype: float """
        return self._global_max

    @property
    def symmetric_scale(self):
        """ :rtype: bool """
        return self._symmetric_scale

    @property
    def samples_unit(self):
        return self._samples_unit

    @samples_unit.setter
    def samples_unit(self, val):
        self._samples_unit = val
        self.context_changed.emit()

    @property
    def image_size(self):
        """ :rtype: None | Tuple(float, float, int) """
        return self._image_size

    def set_colormap(self, colormap):
        self._colormap_name = colormap
        self.context_changed.emit()

    def show_indicators(self, visible):
        self._show_indicators = visible
        self.context_changed.emit()

    def set_symmetric_scale(self, symmetric):
        self._symmetric_scale = symmetric
        self.context_changed.emit()

    def set_interpolation(self, interpolation_name):
        self._interpolation = str(interpolation_name)
        self.context_changed.emit()

    def set_x_view_limits(self, index_direction, min_x, max_x):

        slice_view_dir_ctxt = self._view_limits[index_direction['name']]

        if max_x is not None:
            slice_view_dir_ctxt.max_xlim = max_x

        if min_x is not None:
            slice_view_dir_ctxt.min_xlim = min_x

        self.context_changed.emit()

    def set_y_view_limits(self, index_direction, min_y, max_y):
        view_limit = self._view_limits[index_direction['name']]

        if max_y is not None:
            view_limit.max_ylim = max_y

        if min_y is not None:
            view_limit.min_ylim = min_y

        self.context_changed.emit()

    def set_user_values(self, min_value, max_value):
        dirty = False
        if self._user_min_value != min_value:
            self._user_min_value = min_value
            dirty = True

        if self._user_max_value != max_value:
            self._user_max_value = max_value
            dirty = True

        if dirty:
            self.context_changed.emit()

    def set_image_size(self, width, height=None, dpi=None):
        if width is None:
            self._image_size = None
            return

        if height is None or dpi is None:
            raise ValueError("Internal error: expects width, height, dpi")

        self._image_size = (width, height, dpi)

    def create_context(self, assigned_slice_models):
        view_min = None
        view_max = None

        for model in assigned_slice_models:
            view_min = model.min_value if view_min is None else min(model.min_value, view_min)
            view_max = model.max_value if view_max is None else max(model.max_value, view_max)
            self._global_min = view_min if self._global_min is None else min(self._global_min, view_min)
            self._global_max = view_max if self._global_max is None else max(self._global_max, view_max)

        vmin = self._global_min if self._user_min_value is None else self._user_min_value
        vmax = self._global_max if self._user_max_value is None else self._user_max_value
        if self._symmetric_scale and vmin <= 0.0 <= vmax:
            vmax = max(abs(vmin), vmax)
            vmin = -vmax

        if vmax < vmin:
            vmax = vmin

        return {
            "colormap": self.colormap,
            "show_indicators": self.indicators,
            "global_min": self._global_min,
            "global_max": self._global_max,
            "view_min": view_min,
            "view_max": view_max,
            "min": vmin,
            "max": vmax,
            "interpolation": self.interpolation,
            "view_limits": self._view_limits,
            "samples_unit": self.samples_unit,
        }

    def _assign_indexes(self):
        for m in self._available_slice_models:
            m.indexes = list(self._slice_data_source.indexes_for_direction(m.index_direction))
            m.x_indexes = list(self._slice_data_source.indexes_for_direction(m.x_index_direction))
            m.y_indexes = list(self._slice_data_source.indexes_for_direction(m.y_index_direction))
            data = self._slice_data_source.read_slice(m.index_direction, m.index)
            m.data = data

    def update_index_for_direction(self, index_direction, index):
        """
        :type index_direction: SliceDirection
        :type index: int
        """
        indexes = self._slice_data_source.indexes_for_direction(index_direction)

        if index < 0:
            index = 0
        elif index >= len(indexes):
            index = len(indexes) - 1

        for m in self._available_slice_models:
            if m.index_direction == index_direction:
                m.index = index

            if m.x_index_direction == index_direction:
                m.x_index = index

            if m.y_index_direction == index_direction:
                m.y_index = index

        self.load_data()

    def load_data(self):
        for m in [sm for sm in self._available_slice_models if sm.dirty and sm.visible]:
            # print("loading data for %s" % m.title)
            m.data = self._slice_data_source.read_slice(m.index_direction, m.index)
        self.data_changed.emit()

    def _reset(self):
        self._global_max = None
        self._global_min = None

        for model in self._available_slice_models:
            model.reset()

        self._assign_indexes()
        self.data_source_changed.emit()

    def slice_data_source(self):
        """ :rtype: SliceDataSource"""
        return self._slice_data_source

    def model_for_direction(self, direction):
        for m in self._available_slice_models:
            if m.index_direction == direction:
                return m
        return None

    def index_for_direction(self, direction):
        for m in self._available_slice_models:
            if m.index_direction == direction:
                return m.index
        return -1



from matplotlib import gridspec
from matplotlib.figure import Figure
from matplotlib.axes import Axes


class LayoutFigure(Figure):
    def __init__(self, width=11.7, height=8.3, dpi=100, tight_layout=True, **kwargs):
        super(LayoutFigure, self).__init__(figsize=(width, height), dpi=dpi, tight_layout=tight_layout, **kwargs)

        self._current_layout = None

        self._axes = []
        """ :type: list[Axes] """

        self._colormap_axes = None
        """ :type: Axes """

    def set_plot_layout(self, layout_spec):
        rows, columns = layout_spec['dims']
        width = 0.025
        ratios = [(1.0 - width) / float(columns)] * columns
        ratios.append(width)

        grid_spec = gridspec.GridSpec(rows, columns + 1, width_ratios=ratios)

        for axes in self._axes:
            self.delaxes(axes)
        self._axes = [self.add_subplot(grid_spec[sub_spec]) for sub_spec in layout_spec['grid']]

        if self._colormap_axes is not None:
            self.delaxes(self._colormap_axes)
        self._colormap_axes = self.add_subplot(grid_spec[:, columns])

        self._current_layout = layout_spec

    def index(self, axes):
        """
        :param axes: The Axes instance to find the index of.
        :type axes: Axes
        :rtype: int
        """
        return None if axes is self._colormap_axes else self._axes.index(axes)

    def colormap_axes(self):
        """
        :rtype: Axes
        """
        return self._colormap_axes

    def layout_axes(self):
        """ :rtype: list[Axes] """
        return list(self._axes)

    def current_layout(self):
        return self._current_layout



import os.path as path

img_prefix = path.abspath(path.join(path.dirname(path.abspath(__file__)), "resources", "img"))


def resource_icon_path(name):
    return path.join(img_prefix, name)


def resource_icon(name):
    """Load an image as an icon"""
    # print("Icon used: %s" % name)
    from PyQt4.QtGui import QIcon
    return QIcon(resource_icon_path(name))


from .arrayspinbox import ArraySpinBox
from ._indexcontroller import IndexController
from ._samplescalecontroller import SampleScaleController
from ._plotexportsettingscontroller import PlotExportSettingsWidget

from .colormapcombo import ColormapCombo
from .layoutcombo import LayoutCombo
from .layoutfigure import LayoutFigure
from .layoutcanvas import LayoutCanvas

from .slicemodel import SliceModel, SliceDirection
from .slicedatasource import SliceDataSource
from .sliceviewcontext import SliceViewContext
from .sliceview import SliceView
from .sliceviewwidget import SliceViewWidget
from .settingswindow import SettingsWindow
from .segyviewwidget import SegyViewWidget

from .version import version as __version__

__copyright__ = 'Copyright 2016, Statoil ASA'
__license__ = 'GNU Lesser General Public License version 3'
__status__ = 'Production'



from PyQt4.QtCore import QSize
from PyQt4.QtGui import QComboBox, QPixmap, qRgb
from PyQt4.QtCore import Qt
from PyQt4.QtGui import QImage

import numpy as np
from matplotlib.cm import ScalarMappable


class ColormapCombo(QComboBox):
    def __init__(self, color_maps=None, parent=None):
        QComboBox.__init__(self, parent)
        self.setMaxVisibleItems(10)
        self.setStyleSheet("QComboBox { combobox-popup: 0; }")

        if color_maps is None:
            color_maps = self._type_sorted_color_maps()

        self.setMinimumWidth(170)
        self.setMaximumWidth(170)
        self.setMinimumHeight(30)

        self.setIconSize(QSize(128, 16))

        icon_width = 256
        icon_height = 16
        values = np.linspace(0, 1, icon_width)
        color_indexes = np.linspace(0, 255, icon_width, dtype=np.uint8)
        color_indexes = np.tile(color_indexes, icon_height)
        image = QImage(color_indexes.data, icon_width, icon_height, QImage.Format_Indexed8)

        for index, item in enumerate(color_maps):
            self.addItem(item)
            pix_map = self._create_icon(item, image, values)
            self.setItemData(index, "", Qt.DisplayRole)
            self.setItemData(index, item, Qt.ToolTipRole)
            self.setItemData(index, pix_map, Qt.DecorationRole)

    def _create_icon(self, color_map_name, image, values):
        """"
        :type color_map_name: str
        :type image: QImage
        :type values: np.ndarray
        """

        color_map = ScalarMappable(cmap=color_map_name)
        rgba = color_map.to_rgba(values, bytes=True)

        color_table = [qRgb(c[0], c[1], c[2]) for c in rgba]
        image.setColorTable(color_table)

        return QPixmap.fromImage(image).scaledToWidth(128)

    def _type_sorted_color_maps(self):
        cmaps = [('Perceptually Uniform Sequential', ['viridis', 'inferno', 'plasma', 'magma']),
                 ('Sequential', ['Blues', 'BuGn', 'BuPu',
                                 'GnBu', 'Greens', 'Greys', 'Oranges', 'OrRd',
                                 'PuBu', 'PuBuGn', 'PuRd', 'Purples', 'RdPu',
                                 'Reds', 'YlGn', 'YlGnBu', 'YlOrBr', 'YlOrRd']),
                 ('Sequential (2)', ['afmhot', 'autumn', 'bone', 'cool',
                                     'copper', 'gist_heat', 'gray', 'hot',
                                     'pink', 'spring', 'summer', 'winter']),
                 ('Diverging', ['BrBG', 'bwr', 'coolwarm', 'PiYG', 'PRGn', 'PuOr',
                                'RdBu', 'RdGy', 'RdYlBu', 'RdYlGn', 'Spectral', 'seismic']),
                 ('Qualitative', ['Accent', 'Dark2', 'Paired', 'Pastel1',
                                  'Pastel2', 'Set1', 'Set2', 'Set3']),
                 ('Miscellaneous', ['gist_earth', 'terrain', 'ocean', 'gist_stern',
                                    'brg', 'CMRmap', 'cubehelix',
                                    'gnuplot', 'gnuplot2', 'gist_ncar',
                                    'nipy_spectral', 'jet', 'rainbow',
                                    'gist_rainbow', 'hsv', 'flag', 'prism'])
                 ]

        # color_maps = sorted(m for m in cm.datad if not m.endswith("_r"))

        color_maps = []

        for cm_group in cmaps:
            color_maps.extend(cm_group[1])

        return color_maps

    def itemText(self, index):
        return str(self.itemData(index, Qt.ToolTipRole).toString())



import numpy as np


class SliceDirection(object):
    inline = {'name': "Inlines"}
    crossline = {'name': "Crosslines"}
    depth = {'name': "Depth"}


class SliceModel(object):
    def __init__(self, title, index_direction, x_index_direction, y_index_direction):
        super(SliceModel, self).__init__()
        self._index_direction = index_direction
        self._x_index_direction = x_index_direction
        self._y_index_direction = y_index_direction
        self._title = title
        self._indexes = None
        self._x_indexes = None
        self._y_indexes = None

        self._data = None
        self._data_x_indexes = None
        self._data_y_indexes = None

        self._index = 0
        self._x_index = 0
        self._y_index = 0

        self._min_value = None
        self._max_value = None

        self._visible = True
        self._dirty = False

    @property
    def title(self):
        """ :rtype: str """
        return self._title

    @property
    def indexes(self):
        """ :rtype: list[int] """
        return self._indexes

    @indexes.setter
    def indexes(self, indexes):
        """ :type indexes: list[int] """
        self._indexes = indexes
        self._index = len(indexes) / 2

    @property
    def x_indexes(self):
        """ @rtype: list[int] """
        return self._x_indexes or self._data_x_indexes

    @x_indexes.setter
    def x_indexes(self, indexes):
        """ :type indexes: list[int] """
        self._assert_shape(self._data, indexes, self._y_indexes)
        self._x_indexes = indexes
        self.x_index = len(indexes) / 2

    @property
    def y_indexes(self):
        """ @rtype: list[int] """
        return self._y_indexes or self._data_y_indexes

    @y_indexes.setter
    def y_indexes(self, indexes):
        """ :type indexes: list[int] """
        self._assert_shape(self._data, self._x_indexes, indexes)
        self._y_indexes = indexes
        self.y_index = len(indexes) / 2

    @staticmethod
    def _assert_shape(data, x_indexes, y_indexes):
        if data is not None:
            if x_indexes and len(x_indexes) != data.shape[1]:
                raise ValueError("X axis element count does not match data shape")

            if y_indexes and len(y_indexes) != data.shape[0]:
                raise ValueError("Y axis element count does not match data shape")

    @property
    def x_axis_name(self):
        """ @rtype: str """
        return self._x_index_direction['name']

    @property
    def y_axis_name(self):
        """ @rtype: str """
        return self._y_index_direction['name']

    @property
    def data(self):
        """ :rtype: numpy.ndarray """
        return self._data

    @data.setter
    def data(self, data):
        """ :type: numppy.ndarray """
        self._assert_shape(data, self._x_indexes, self._y_indexes)
        data[data == -np.inf] = 0.0
        data[data == np.inf] = 0.0
        self._data = data
        self._min_value = np.nanmin(self.data)
        self._max_value = np.nanmax(self.data)
        self._data_x_indexes = list(range(data.shape[0]))
        self._data_y_indexes = list(range(data.shape[1]))
        self._dirty = False

    @property
    def index(self):
        """ :rtype: int """
        return self._index

    @index.setter
    def index(self, index):
        """ :type index: int """
        if self._index != index:
            self._dirty = True
        self._index = index

    @property
    def min_x(self):
        """ :rtype: int """
        return min(self.x_indexes)

    @property
    def max_x(self):
        """ :rtype: int """
        return max(self.x_indexes)

    @property
    def min_y(self):
        """ :rtype: int """
        return min(self.y_indexes)

    @property
    def max_y(self):
        """ :rtype: int """
        return max(self.y_indexes)

    @property
    def min_value(self):
        """ :rtype: int """
        return self._min_value

    @property
    def max_value(self):
        """ :rtype: int """
        return self._max_value

    def x_axis_formatter(self, value, position):
        if 0 <= value < self.width:
            return self.x_indexes[int(value)]
        return ''

    def y_axis_formatter(self, value, position):
        if 0 <= value < self.height:
            return self.y_indexes[int(value)]
        return ''

    @property
    def x_index(self):
        """ :rtype: int """
        return self._x_index

    @x_index.setter
    def x_index(self, index):
        """ :type index: int """
        self._x_index = index

    @property
    def y_index(self):
        """ :rtype: int """
        return self._y_index

    @y_index.setter
    def y_index(self, index):
        """ :type index: int """
        self._y_index = index

    @property
    def width(self):
        """ :rtype: int """
        return len(self.x_indexes)

    @property
    def height(self):
        """ :rtype: int """
        return len(self.y_indexes)

    @property
    def visible(self):
        """ :rtype: bool """
        return self._visible

    @visible.setter
    def visible(self, visible):
        """ :type visible: bool """
        self._visible = visible

    @property
    def index_direction(self):
        """ :rtype: dict """
        return self._index_direction

    @property
    def x_index_direction(self):
        """ :rtype: dict """
        return self._x_index_direction

    @property
    def y_index_direction(self):
        """ :rtype: dict """
        return self._y_index_direction

    @property
    def dirty(self):
        """ :type: bool """
        return self._dirty

    def __len__(self):
        return len(self.indexes)

    def reset(self):
        self._indexes = None
        self._x_indexes = None
        self._y_indexes = None

        self._data = None
        self._data_x_indexes = None
        self._data_y_indexes = None

        self._index = 0
        self._x_index = 0
        self._y_index = 0

        self._min_value = None
        self._max_value = None



from unittest import TestCase
from segyviewlib import SliceViewContext, SliceModel, SliceDirection as SD, SliceDataSource


class SliceViewContextTest(TestCase):
    @staticmethod
    def _generate_mock_model_and_context():
        model = SliceModel("test", SD.inline, SD.crossline, SD.depth)
        svc = SliceViewContext([model], SliceDataSource(None))
        return model, svc

    def test_min_max_symmetry(self):
        model, svc = SliceViewContextTest._generate_mock_model_and_context()

        model._min_value = -20
        model._max_value = 120
        context = svc.create_context([model])

        # when min<0<max, expect the scale to be symmetrically centered around 0
        self.assertEqual(context['min'], -120)
        self.assertEqual(context['max'], 120)

    def test_min_max_pos_values(self):
        model, svc = SliceViewContextTest._generate_mock_model_and_context()

        model._min_value = 1
        model._max_value = 120
        context = svc.create_context([model])

        self.assertEqual(context['min'], 1)
        self.assertEqual(context['max'], 120)

    def test_min_max_neg_values(self):
        model, svc = SliceViewContextTest._generate_mock_model_and_context()

        model._min_value = -40
        model._max_value = -21
        context = svc.create_context([model])

        self.assertEqual(context['min'], -40)
        self.assertEqual(context['max'], -21)



from matplotlib import patches
from matplotlib.ticker import FuncFormatter, AutoLocator
from matplotlib.image import AxesImage
from .slicemodel import SliceModel, SliceDirection
import numpy as np


class SliceView(object):
    def __init__(self, axes, model, interpolation="nearest", aspect="auto"):
        super(SliceView, self).__init__()
        data = np.zeros((1, 1))
        self._image = axes.imshow(data, interpolation=interpolation, aspect=aspect, origin='upper')
        """ :type: AxesImage """
        self._model = model
        """ :type: SliceModel """

        style = {"fill": False,
                 "alpha": 1,
                 "color": 'black',
                 "linestyle": 'dotted',
                 "linewidth": 0.75
                 }

        self._vertical_indicator = patches.Rectangle((-0.5, -0.5), 1, model.height, **style)
        self._horizontal_indicator = patches.Rectangle((-0.5, -0.5), model.width, 1, **style)
        self._zoom_factor = 1.0

        self._view_limits = None

        self._min_xlim = 0
        self._max_xlim = model.width
        self._min_ylim = 0
        self._max_ylim = model.height

    def model(self):
        """ :rtype: SliceModel """
        return self._model

    def create_slice(self, context):
        """ :type context: dict """
        model = self._model
        axes = self._image.axes
        """ :type: matplotlib.axes.Axes """

        axes.set_title(model.title, fontsize=12)
        axes.tick_params(axis='both')
        axes.set_ylabel(model.y_axis_name, fontsize=9)
        axes.set_xlabel(model.x_axis_name, fontsize=9)

        axes.get_xaxis().set_major_formatter(FuncFormatter(model.x_axis_formatter))
        axes.get_xaxis().set_major_locator(AutoLocator())

        axes.get_yaxis().set_major_formatter(FuncFormatter(model.y_axis_formatter))
        axes.get_yaxis().set_major_locator(AutoLocator())

        for label in (axes.get_xticklabels() + axes.get_yticklabels()):
            label.set_fontsize(9)

        self._reset_zoom()

        axes.add_patch(self._vertical_indicator)
        axes.add_patch(self._horizontal_indicator)
        self._update_indicators(context)

        self._image.set_cmap(cmap=context['colormap'])

        self._view_limits = context["view_limits"][self._model.index_direction['name']]

        if model.data is not None:
            self._image.set_data(model.data)

    def data_changed(self, context):
        """ :type context: dict """
        model = self._model
        self._image.set_data(model.data)
        self._image.set_extent((0, model.width, model.height, 0))

    def context_changed(self, context):
        """ :type context: dict """
        self._image.set_cmap(context['colormap'])
        self._image.set_clim(context['min'], context['max'])
        self._image.set_interpolation(context['interpolation'])
        self._update_indicators(context)

        self._set_view_limits()

        if self._model.index_direction is not SliceDirection.depth:
            self._image.axes.set_ylabel(context['samples_unit'])

    def _update_indicators(self, context):
        """ :type context: dict """
        model = self._model
        self._vertical_indicator.set_height(model.height + 1)
        self._vertical_indicator.set_width(1)
        self._horizontal_indicator.set_width(model.width + 1)
        self._horizontal_indicator.set_height(1)

        show_indicators = context['show_indicators']

        self._vertical_indicator.set_visible(model.x_index is not None and show_indicators)
        self._horizontal_indicator.set_visible(model.y_index is not None and show_indicators)

        if model.x_index is not None and show_indicators:
            self._vertical_indicator.set_x(model.x_index)

        if model.y_index is not None and show_indicators:
            self._horizontal_indicator.set_y(model.y_index)

    def _has_view_limits_changed(self):
        return any([self._view_limits.min_xlim != self._min_xlim,
                    self._view_limits.max_xlim != self._max_xlim,
                    self._view_limits.min_ylim != self._min_ylim,
                    self._view_limits.max_ylim != self._max_ylim])

    def _set_view_limits(self, forced_reset=False):

        if self._has_view_limits_changed() or forced_reset:
            self._reset_zoom()

            axes = self._image.axes

            self._min_xlim = self._view_limits.min_xlim
            self._max_xlim = self._view_limits.max_xlim
            axes.set_xlim(self._min_xlim, self._max_xlim)

            self._min_ylim = self._view_limits.min_ylim
            self._max_ylim = self._view_limits.max_ylim
            axes.set_ylim(self._max_ylim, self._min_ylim)  # flipping since y axis is flipped

    def _reset_zoom(self):
        self._zoom_factor = 1

    def zoom(self, x, y, zoom_factor_delta, reset_zoom=False):
        zoom_factor = self._zoom_factor + zoom_factor_delta

        if zoom_factor > 1.0:
            zoom_factor = 1.0
            reset_zoom = True
        elif zoom_factor < 0.25:
            zoom_factor = 0.25

        if abs(zoom_factor - self._zoom_factor) < 0.001:
            return False

        self._zoom_factor = zoom_factor

        if reset_zoom:
            self._set_view_limits(forced_reset=True)
            return True

        model = self._model
        axes = self._image.axes

        new_width = (self._max_xlim - self._min_xlim) * zoom_factor
        new_height = (self._max_ylim - self._min_ylim) * zoom_factor

        cur_xlim = axes.get_xlim()
        cur_ylim = axes.get_ylim()

        relx = (cur_xlim[1] - x) / (cur_xlim[1] - cur_xlim[0])
        rely = (cur_ylim[0] - y) / (cur_ylim[0] - cur_ylim[1])

        min_zoom_xlim = max(max(0, x - new_width * (1 - relx)), self._min_xlim)
        max_zoom_xlim = min(min(model.width, x + new_width * relx), self._max_xlim)

        max_zoom_ylim = min(min(model.height, y + new_height * rely), self._max_ylim)
        min_zoom_ylim = max(max(0, y - new_height * (1 - rely)), self._min_ylim)

        axes.set_xlim(min_zoom_xlim, max_zoom_xlim)
        axes.set_ylim(max_zoom_ylim, min_zoom_ylim)
        return True

    def pan(self, dx, dy):
        axes = self._image.axes

        xlim = axes.get_xlim()

        if xlim[0] - dx < self._min_xlim:
            dx = xlim[0] - self._min_xlim
        elif xlim[1] - dx > self._max_xlim:
            dx = xlim[1] - self._max_xlim

        ylim = axes.get_ylim()

        if ylim[1] - dy < self._min_ylim:
            dy = ylim[1] - self._min_ylim
        elif ylim[0] - dy > self._max_ylim:
            dy = ylim[0] - self._max_ylim

        axes.set_xlim(xlim - dx)
        axes.set_ylim(ylim - dy)



from PyQt4.QtCore import Qt, pyqtSignal, QVariant
from PyQt4.QtGui import QComboBox, QIcon

from segyviewlib import resource_icon


class LayoutCombo(QComboBox):
    layout_changed = pyqtSignal(object)

    DEFAULT_LARGE_FILE_LAYOUT = 4
    DEFAULT_SMALL_FILE_LAYOUT = 2

    def __init__(self, parent=None):
        QComboBox.__init__(self, parent)

        layouts = [
            # {
            #     "icon": "layouts_four_grid.png",
            #     "spec": {
            #         "dims": (2, 2),
            #         "grid": [(0, 0), (0, 1), (1, 0), (1, 1)]
            #     }
            # },
            {
                "icon": "layouts_three_bottom_grid.png",
                "spec": {
                    "dims": (2, 2),
                    "grid": [(0, 0), (0, 1), (1, slice(0, 2))]
                }
            },
            {
                "icon": "layouts_three_top_grid.png",
                "spec": {
                    "dims": (2, 2),
                    "grid": [(0, slice(0, 2)), (1, 0), (1, 1)]
                }
            },
            {
                "icon": "layouts_three_left_grid.png",
                "spec": {
                    "dims": (2, 2),
                    "grid": [(slice(0, 2), 0), (0, 1), (1, 1)]
                }
            },
            {
                "icon": "layouts_three_right_grid.png",
                "spec": {
                    "dims": (2, 2),
                    "grid": [(0, 0), (1, 0), (slice(0, 2), 1)]
                }
            },
            {
                "icon": "layouts_two_horizontal_grid.png",
                "spec": {
                    "dims": (2, 1),
                    "grid": [(0, 0), (1, 0)]
                }
            },
            {
                "icon": "layouts_two_vertical_grid.png",
                "spec": {
                    "dims": (1, 2),
                    "grid": [(0, 0), (0, 1)]
                }
            },
            {
                "icon": "layouts_three_horizontal_grid.png",
                "spec": {
                    "dims": (3, 1),
                    "grid": [(0, 0), (1, 0), (2, 0)]
                }
            },
            {
                "icon": "layouts_three_vertical_grid.png",
                "spec": {
                    "dims": (1, 3),
                    "grid": [(0, 0), (0, 1), (0, 2)]
                }
            },
            {
                "icon": "layouts_single.png",
                "spec": {
                    "dims": (1, 1),
                    "grid": [(0, 0)]
                }
            }
        ]

        for layout in layouts:
            self.add_layout_item(layout)

        height = 30
        self.setMinimumHeight(height)
        self.setMaximumHeight(height)
        self.setMinimumWidth(60)
        self.setMaximumWidth(60)

        self.currentIndexChanged.connect(self._layout_changed)

    def add_layout_item(self, layout_item):
        self.addItem(resource_icon(layout_item['icon']), "", layout_item['spec'])

    def _layout_changed(self, index):
        spec = self._get_spec(index)
        self.layout_changed.emit(spec)

    def _get_spec(self, index):
        user_data = self.itemData(index)
        """ :type: QVariant"""
        spec = user_data.toPyObject()
        spec = {str(key): value for key, value in spec.items()}
        return spec

    def get_current_layout(self):
        return self._get_spec(self.currentIndex())



from PyQt4.QtGui import QCheckBox, QWidget, QLabel
from PyQt4.QtCore import Qt, pyqtSignal, QObject

from segyviewlib import ArraySpinBox


class IndexController(QObject):
    index_changed = pyqtSignal(int)
    min_max_changed = pyqtSignal(tuple)

    def __init__(self, parent=None, context=None, slice_direction_index_source=None):
        QObject.__init__(self, parent)

        self._context = context
        self._slice_direction = slice_direction_index_source

        self._current_index_label = QLabel("")
        self._current_index_label.setDisabled(True)

        # the select-index-widgets
        self.index_s_box = self._set_up_index_widget()

        # the min max settings widgets
        self._min_active, self._min_spinbox = self._set_up_min_max_widgets()
        self._max_active, self._max_spinbox = self._set_up_min_max_widgets()

        self._initialize(self._context.slice_data_source().indexes_for_direction(self._slice_direction).tolist())

    @property
    def index_widget(self):
        return self.index_s_box

    @property
    def min_checkbox(self):
        return self._min_active

    @property
    def min_spinbox(self):
        return self._min_spinbox

    @property
    def max_checkbox(self):
        return self._max_active

    @property
    def max_spinbox(self):
        return self._max_spinbox

    @property
    def current_index_label(self):
        return self._current_index_label

    def _set_up_index_widget(self):
        index_s_box = ArraySpinBox([0])
        index_s_box.valueChanged.connect(self.index_changed.emit)
        return index_s_box

    def _set_up_min_max_widgets(self):
        check_box = QCheckBox()
        spin_box = ArraySpinBox([0])
        spin_box.setDisabled(True)

        check_box.toggled.connect(spin_box.setEnabled)
        check_box.toggled.connect(self._min_max_value_changed)
        spin_box.valueChanged.connect(self._min_max_value_changed)

        return check_box, spin_box

    def _initialize(self, indexes):
        self._indexes = indexes
        # set up initial min max values
        self.current_index = 0
        self.min_index = 0
        self.max_index = len(self._indexes) - 1

        self.index_s_box.update_view(self._indexes, self.current_index)

        self._min_spinbox.update_view(self._indexes, self.min_index)
        self._max_spinbox.update_view(self._indexes, self.max_index)

        self._min_spinbox.setMaximum(self.max_index - 1)
        self._max_spinbox.setMinimum(self.min_index + 1)

        self._max_active.setCheckState(Qt.Unchecked)
        self._min_active.setCheckState(Qt.Unchecked)

        self._update_label()
        self._min_max_value_changed()

    @staticmethod
    def _to_lbl_txt(val):
        return str(round(val, 4)) if isinstance(val, float) else str(val)

    def _update_label(self):
        self._current_index_label.setText(
            "pos: {0} - [{1}:{2}]".format(self._to_lbl_txt(self._indexes[self.current_index]),
                                          self._to_lbl_txt(self._indexes[self.min_index]),
                                          self._to_lbl_txt(self._indexes[self.max_index])))

    def update_index(self, index):
        self.index_s_box.update_view(self._indexes, index)
        self.current_index = index
        self._update_label()

    def update_view(self, indexes, index):
        if self._indexes != indexes:
            # re initialize when index is changed
            self._initialize(indexes)
        else:
            self.update_index(index)

    def _min_max_value_changed(self):
        if self._min_active.isChecked():
            self.min_index = self._min_spinbox.value()
            min_max = self.min_index + 1 if self.min_index < self.max_index - 1 else self.max_index
            self._max_spinbox.setMinimum(min_max)

            # move current index to inside the boundary
            if self.min_index > self.current_index:
                self.index_changed.emit(self.min_index)
        else:
            self.min_index = 0

        if self._max_active.isChecked():
            self.max_index = self._max_spinbox.value()
            max_min = self.max_index - 1 if self.max_index > 0 else 0
            self._min_spinbox.setMaximum(max_min)

            # move current index to inside the boundary
            if self.max_index < self.current_index:
                self.index_changed.emit(self.max_index)
        else:
            self.max_index = len(self._indexes) - 1

        self.min_max_changed.emit((self.min_index, self.max_index))



from PyQt4.QtGui import QFileDialog, QToolButton, QToolBar, QVBoxLayout, QWidget

from segyviewlib import ColormapCombo, LayoutCombo, SettingsWindow, SliceViewContext
from segyviewlib import SliceDataSource, SliceModel, SliceDirection as SD, SliceViewWidget, resource_icon


class SegyViewWidget(QWidget):
    def __init__(self, filename, show_toolbar=True, color_maps=None,
                                 width=11.7, height=8.3, dpi=100,
                                 segyioargs = {}, parent=None):
        QWidget.__init__(self, parent)

        inline = SliceModel("Inline", SD.inline, SD.crossline, SD.depth)
        xline = SliceModel("Crossline", SD.crossline, SD.inline, SD.depth)
        depth = SliceModel("Depth", SD.depth, SD.inline, SD.crossline)

        slice_models = [inline, xline, depth]
        slice_data_source = SliceDataSource(filename, **segyioargs)
        self._slice_data_source = slice_data_source

        self._context = SliceViewContext(slice_models, slice_data_source)
        self._context.show_indicators(True)

        self._slice_view_widget = SliceViewWidget(self._context, width, height, dpi, self)

        layout = QVBoxLayout()

        self._settings_window = SettingsWindow(self._context, self)

        self._toolbar = self._create_toolbar(color_maps)
        self._toolbar.setVisible(show_toolbar)
        layout.addWidget(self._toolbar)
        layout.addWidget(self._slice_view_widget)

        self.setLayout(layout)

    def toolbar(self):
        """ :rtype: QToolBar """
        return self._toolbar

    def _create_toolbar(self, color_maps):
        toolbar = QToolBar()
        toolbar.setFloatable(False)
        toolbar.setMovable(False)

        self.layout_combo = LayoutCombo()
        toolbar.addWidget(self.layout_combo)
        self.layout_combo.layout_changed.connect(self._slice_view_widget.set_plot_layout)

        # self._colormap_combo = ColormapCombo(['seismic', 'spectral', 'RdGy', 'hot', 'jet', 'gray'])
        self._colormap_combo = ColormapCombo(color_maps)
        self._colormap_combo.currentIndexChanged[int].connect(self._colormap_changed)
        toolbar.addWidget(self._colormap_combo)

        save_button = QToolButton()
        save_button.setToolTip("Save as image")
        save_button.setIcon(resource_icon("table_export.png"))
        save_button.clicked.connect(self._save_figure)
        toolbar.addWidget(save_button)

        self._settings_button = QToolButton()
        self._settings_button.setToolTip("Toggle settings visibility")
        self._settings_button.setIcon(resource_icon("cog.png"))
        self._settings_button.setCheckable(True)
        self._settings_button.toggled.connect(self._show_settings)
        toolbar.addWidget(self._settings_button)

        def toggle_on_close(event):
            self._settings_button.setChecked(False)
            event.accept()

        self._settings_window.closeEvent = toggle_on_close

        self._colormap_combo.setCurrentIndex(45)
        self.set_default_layout()

        return toolbar

    def _colormap_changed(self, index):
        colormap = str(self._colormap_combo.itemText(index))
        self._context.set_colormap(colormap)

    def _interpolation_changed(self, index):
        interpolation_name = str(self._interpolation_combo.itemText(index))
        self._context.set_interpolation(interpolation_name)

    def _save_figure(self):
        formats = "Portable Network Graphic (*.png);;Adobe Acrobat (*.pdf);;Scalable Vector Graphics (*.svg)"
        output_file = QFileDialog.getSaveFileName(self, "Save as image", "untitled.png", formats)

        output_file = str(output_file).strip()

        if len(output_file) == 0:
            return

        image_size = self._context.image_size
        if not image_size:
            fig = self._slice_view_widget
        else:
            w, h, dpi = image_size
            fig = SliceViewWidget(self._context, width = w, height = h, dpi = dpi)
            fig.set_plot_layout(self._slice_view_widget.layout_figure().current_layout())

        fig.layout_figure().savefig(output_file)

    def set_source_filename(self, filename):
        self._slice_data_source.set_source_filename(filename)

    def set_default_layout(self):
        # default slice view layout depends on the file size
        if self._slice_data_source.file_size < 8 * 10 ** 8:
            self.layout_combo.setCurrentIndex(self.layout_combo.DEFAULT_SMALL_FILE_LAYOUT)
        else:
            self.layout_combo.setCurrentIndex(self.layout_combo.DEFAULT_LARGE_FILE_LAYOUT)

    def as_depth(self):
        self._context.samples_unit = 'Depth (m)'

    def _show_settings(self, toggled):
        self._settings_window.setVisible(toggled)
        if self._settings_window.isMinimized():
            self._settings_window.showNormal()



from PyQt4 import QtGui
from PyQt4.QtCore import pyqtSignal, Qt

from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas

from segyviewlib import LayoutFigure


class Keys(object):
    def __init__(self, key=None, ctrl=False, alt=False, shift=False, meta=False):
        super(Keys, self).__init__()
        self.key = key
        self.alt = alt
        self.ctrl = ctrl
        self.super = meta
        self.shift = shift

    def state(self, ctrl=False, alt=False, shift=False, meta=False):
        return self.ctrl == ctrl and self.alt == alt and self.shift == shift and self.super == meta

    def __bool__(self):
        return not (self.key is None and not self.alt and not self.ctrl and not self.super and not self.shift)

    __nonzero__ = __bool__

    def __str__(self):
        return "%s ctrl: %s shift: %s alt: %s super: %s self: %s" % (self.key, self.ctrl, self.shift, self.alt, self.super, bool(self))


class LayoutCanvas(FigureCanvas):
    layout_changed = pyqtSignal()
    subplot_pressed = pyqtSignal(dict)
    subplot_released = pyqtSignal(dict)
    subplot_motion = pyqtSignal(dict)
    subplot_scrolled = pyqtSignal(dict)

    def __init__(self, width=11.7, height=8.3, dpi=100, parent=None):
        self._figure = LayoutFigure(width, height, dpi)

        FigureCanvas.__init__(self, self._figure)
        self.setParent(parent)

        self.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        self.updateGeometry()
        self.setFocusPolicy(Qt.WheelFocus)
        self.setFocus()

        self.mpl_connect('scroll_event', self._mouse_scrolled)
        self.mpl_connect('button_press_event', self._mouse_pressed)
        self.mpl_connect('button_release_event', self._mouse_released)
        self.mpl_connect('motion_notify_event', self._mouse_motion)
        self.mpl_connect('key_press_event', self._key_press_event)
        self.mpl_connect('key_release_event', self._key_release_event)

        self._start_x = None
        self._start_y = None
        self._start_subplot_index = None
        self._keys = Keys()

    def _create_event(self, event):
        data = {
            "x": event.xdata,
            "y": event.ydata,
            "mx": event.x,
            "my": event.y,
            "dx": None if self._start_x is None else event.xdata - self._start_x,
            "dy": None if self._start_y is None else event.ydata - self._start_y,
            "button": event.button,
            "key": self._keys,
            "step": event.step,
            "subplot_index": self._start_subplot_index or self._figure.index(event.inaxes),
            "gui_event": event.guiEvent
        }
        return data

    def _key_press_event(self, event):
        if len(event.key) > 0:
            self._keys.ctrl = 'ctrl' in event.key or 'control' in event.key
            self._keys.shift = 'shift' in event.key
            self._keys.alt = 'alt' in event.key
            self._keys.meta = 'super' in event.key
            keys = event.key.replace('ctrl', '')
            keys = keys.replace('control', '')
            keys = keys.replace('alt', '')
            keys = keys.replace('super', '')
            keys = keys.replace('+', '')
            self._keys.keys = keys

    def _key_release_event(self, event):
        # the event is unclear on what key is actually released -> wiping every key :)
        self._keys.keys = None
        self._keys.ctrl = False
        self._keys.alt = False
        self._keys.shift = False
        self._keys.super = False

    def _mouse_scrolled(self, event):
        if event.inaxes is not None:
            data = self._create_event(event)
            self.subplot_scrolled.emit(data)

    def _mouse_pressed(self, event):
        if event.inaxes is not None:
            data = self._create_event(event)
            self._start_x = data['x']
            self._start_y = data['y']
            self._start_subplot_index = data['subplot_index']
            self.subplot_pressed.emit(data)

    def _mouse_released(self, event):
        if event.inaxes is not None:
            self._start_x = None
            self._start_y = None
            self._start_subplot_index = None
            data = self._create_event(event)
            self.subplot_released.emit(data)

    def _mouse_motion(self, event):
        if event.inaxes is not None and self._figure.index(event.inaxes) is not None:
            data = self._create_event(event)
            self.subplot_motion.emit(data)

    def set_plot_layout(self, layout_spec):
        self._figure.set_plot_layout(layout_spec)
        self.layout_changed.emit()
        self.draw()

    def layout_figure(self):
        """ :rtype: LayoutFigure """
        return self._figure



from __future__ import division
from PyQt4.QtGui import QCheckBox, QWidget, QFormLayout, QComboBox, QLabel
from PyQt4.QtGui import QPushButton, QHBoxLayout, QVBoxLayout, QTreeWidget, QTreeWidgetItem
from PyQt4.QtCore import Qt, QObject

from segyviewlib import SliceDirection, SampleScaleController, IndexController, PlotExportSettingsWidget


class SettingsWindow(QWidget):
    def __init__(self, context, parent=None):
        """
        :type context: segyviewlib.SliceViewContext
        :type parent: QObject
        """
        QWidget.__init__(self, parent, Qt.WindowStaysOnTopHint | Qt.Window)
        self.setVisible(False)

        self._context = context
        self._context.context_changed.connect(self._settings_changed)
        self._context.data_changed.connect(self._settings_changed)
        self._context.data_source_changed.connect(self._settings_changed)

        f_layout = QFormLayout()

        self._iline_count = QLabel("")
        self._xline_count = QLabel("")
        self._offset_count = QLabel("")
        self._sample_count = QLabel("")
        self._minimum_value = QLabel("")
        self._maximum_value = QLabel("")

        f_layout.addRow("Inline Count:", self._iline_count)
        f_layout.addRow("Crossline Count:", self._xline_count)
        f_layout.addRow("Offset Count:", self._offset_count)
        f_layout.addRow("Sample Count:", self._sample_count)
        f_layout.addRow("Minimum Value:", self._minimum_value)
        f_layout.addRow("Maximum Value:", self._maximum_value)

        # iline
        self._il_ctrl = IndexController(parent=self,
                                        context=self._context,
                                        slice_direction_index_source=SliceDirection.inline)
        self._il_ctrl.index_changed.connect(self._index_changed_fn(SliceDirection.inline))
        self._il_ctrl.min_max_changed.connect(self.iline_limits_changed)

        # xline
        self._xl_ctrl = IndexController(parent=self,
                                        context=self._context,
                                        slice_direction_index_source=SliceDirection.crossline)
        self._xl_ctrl.index_changed.connect(self._index_changed_fn(SliceDirection.crossline))
        self._xl_ctrl.min_max_changed.connect(self.xline_limits_changed)

        # depth
        self._depth_ctrl = IndexController(parent=self,
                                           context=self._context,
                                           slice_direction_index_source=SliceDirection.depth)
        self._depth_ctrl.index_changed.connect(self._index_changed_fn(SliceDirection.depth))
        self._depth_ctrl.min_max_changed.connect(self.depth_limits_changed)

        # sample
        self._sample_ctrl = SampleScaleController(self)
        self._sample_ctrl.min_max_changed.connect(self.sample_limits_changed)

        self._symmetric_scale = QCheckBox()
        self._symmetric_scale.toggled.connect(self._context.set_symmetric_scale)

        self._samples_unit = QComboBox()
        self._samples_unit.addItems(['Time (ms)', 'Depth (m)'])
        self._samples_unit.currentIndexChanged[str].connect(self.samples_unit)

        # view
        self._view_label = QLabel("")
        self._view_label.setDisabled(True)
        self._indicator_visibility = QCheckBox()
        self._indicator_visibility.toggled.connect(self._context.show_indicators)
        self._indicator_visibility.toggled.connect(lambda: self._set_view_label(self._indicator_visibility.isChecked()))

        self._interpolation_combo = QComboBox()
        self._interpolations_names = ['nearest', 'catrom', 'sinc']
        self._interpolation_combo.addItems(self._interpolations_names)
        self._interpolation_combo.currentIndexChanged.connect(self._interpolation_changed)

        # plot export settings
        self._plt_settings_wdgt = PlotExportSettingsWidget(self, parent._slice_view_widget, self._context)

        # define tree layout
        tree_def = {"": [
            {"Inline": [{"set_expanded": True},
                        {"": self._align(self._il_ctrl.current_index_label)},
                        {"Inline:": self._align(self._il_ctrl.index_widget)},
                        {"Minimum:": self._align(self._il_ctrl.min_spinbox, self._il_ctrl.min_checkbox)},
                        {"Maximum:": self._align(self._il_ctrl.max_spinbox, self._il_ctrl.max_checkbox)}
                        ]
             },
            {"Crossline": [{"set_expanded": True},
                           {"": self._align(self._xl_ctrl.current_index_label)},
                           {"Crossline:": self._align(self._xl_ctrl.index_widget)},
                           {"Minimum:": self._align(self._xl_ctrl.min_spinbox, self._xl_ctrl.min_checkbox)},
                           {"Maximum:": self._align(self._xl_ctrl.max_spinbox, self._xl_ctrl.max_checkbox)}
                           ]
             },
            {"Depth": [{"set_expanded": True},
                       {"": self._align(self._depth_ctrl.current_index_label)},
                       {"Depth:": self._align(self._depth_ctrl.index_widget)},
                       {"Minimum:": self._align(self._depth_ctrl.min_spinbox, self._depth_ctrl.min_checkbox)},
                       {"Maximum:": self._align(self._depth_ctrl.max_spinbox, self._depth_ctrl.max_checkbox)},
                       {"Type": self._align(self._samples_unit)}
                       ]
             },
            {"Color Scale": [
                {"Custom min.:": self._align(self._sample_ctrl.min_spinbox, self._sample_ctrl.min_checkbox)},
                {"Custom max.:": self._align(self._sample_ctrl.max_spinbox, self._sample_ctrl.max_checkbox)},
                {"Symmetric scale:": self._align(self._symmetric_scale)}
            ]
            },
            {"View": [{"": self._align(self._view_label)},
                      {"Show Indicators:": self._align(self._indicator_visibility)},
                      {"Interpolation Type:": self._align(self._interpolation_combo)}
                      ]
             },
            {"Plot export dimensions": [
                {"": self._align(self._plt_settings_wdgt.label)},
                {"Fixed size": self._align(self._plt_settings_wdgt.checkbox)},
                {"Width:": self._align(self._plt_settings_wdgt.width_spinbox)},
                {"Height:": self._align(self._plt_settings_wdgt.height_spinbox)},
                {"Units:": self._align(self._plt_settings_wdgt.units_combobox)}
            ]
            }
        ]}

        # setup the menu/navigation tree widget
        tre = QTreeWidget(self)
        tre.setHeaderHidden(True)
        tre.setColumnCount(2)
        tre.setColumnWidth(0, 140)
        tre.setColumnWidth(1, 180)

        self._build_tree(tre, tree_def, tre.invisibleRootItem())

        # layout
        vertical_layout = QVBoxLayout()
        button_layout = QHBoxLayout()
        button_layout.setContentsMargins(0, 0, 0, 0)
        close_button = QPushButton("Close")
        close_button.clicked.connect(self.close)
        button_layout.addStretch()
        button_layout.addWidget(close_button)

        vertical_layout.addLayout(f_layout, 0)
        vertical_layout.addStretch()
        vertical_layout.addWidget(tre, 1)
        vertical_layout.addStretch()
        vertical_layout.addLayout(button_layout, 0)

        self.setLayout(vertical_layout)
        self.setMinimumSize(390, 740)

    @staticmethod
    def _align(widget, checkbox=None):

        l = QHBoxLayout()

        if checkbox is not None:
            checkbox.setMaximumWidth(23)
            l.addWidget(checkbox, 0)
        else:
            l.addSpacing(25)

        l.addStretch(0.5)
        if widget is not None:
            widget.setMinimumWidth(180)
            widget.setMaximumWidth(180)
            l.addWidget(widget)
        else:
            l.addSpacing(180)

        l.setContentsMargins(0, 0, 0, 0)
        l.addStretch(1)

        w = QWidget()
        w.setContentsMargins(0, 2, 0, 2)
        w.setLayout(l)
        return w

    def samples_unit(self, val):
        self._context.samples_unit = val

    def _create_user_value(self):
        layout = QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

    def _build_tree(self, tree_wdgt, tree_def, root):
        parent, children = tree_def.items()[0]

        # empty label /parent is a special case: either inline with the previous, or skip
        if parent == "":
            if isinstance(children, QWidget):
                item = root
                tree_wdgt.setItemWidget(item, 1, children)

            elif isinstance(children, list):
                for c in children:
                    self._build_tree(tree_wdgt, c, root)

        elif parent == "set_expanded":  # a configuration item for the current root
            root.setExpanded(children)
        else:
            item = QTreeWidgetItem(root)
            item.setText(0, parent)

            if isinstance(children, list):
                for c in children:
                    self._build_tree(tree_wdgt, c, item)
            else:
                tree_wdgt.setItemWidget(item, 1, children)

    def add_empty_row(self, layout, legend=""):
        layout.addRow(legend, QLabel())

    def _settings_changed(self):
        ctx = self._context
        self._indicator_visibility.setChecked(ctx.indicators)

        index = self._interpolations_names.index(ctx.interpolation)
        self._interpolation_combo.setCurrentIndex(index)

        self._symmetric_scale.setChecked(ctx.symmetric_scale)

        ilines, xlines, offsets, samples = ctx.slice_data_source().dims()
        self._iline_count.setText("%d" % ilines)
        self._xline_count.setText("%d" % xlines)
        self._offset_count.setText("%d" % offsets)
        self._sample_count.setText("%d" % samples)

        self._minimum_value.setText("%f" % ctx.global_minimum)
        self._maximum_value.setText("%f" % ctx.global_maximum)

        indexes = ctx.slice_data_source().indexes_for_direction(SliceDirection.inline).tolist()
        index = ctx.index_for_direction(SliceDirection.inline)
        self._il_ctrl.update_view(indexes, index)

        indexes = ctx.slice_data_source().indexes_for_direction(SliceDirection.crossline).tolist()
        index = ctx.index_for_direction(SliceDirection.crossline)
        self._xl_ctrl.update_view(indexes, index)

        indexes = ctx.slice_data_source().indexes_for_direction(SliceDirection.depth).tolist()
        index = ctx.index_for_direction(SliceDirection.depth)
        self._depth_ctrl.update_view(indexes, index)

        index = self._samples_unit.findText(ctx.samples_unit)
        if index != -1:
            self._samples_unit.setCurrentIndex(index)

    def _set_view_label(self, indicator_on):
        self._view_label.setText("indicators {0}".format("on" if indicator_on else "off"))

    def _interpolation_changed(self, index):
        interpolation_name = str(self._interpolation_combo.itemText(index))
        self._context.set_interpolation(interpolation_name)

    def _index_changed_fn(self, direction):
        def fn(value):
            self._context.update_index_for_direction(direction, value)

        return fn

    def sample_limits_changed(self, values):
        self._context.set_user_values(*values)

    def depth_limits_changed(self, values):
        min, max = values
        self._context.set_y_view_limits(SliceDirection.crossline, min, max)
        self._context.set_y_view_limits(SliceDirection.inline, min, max)

    def iline_limits_changed(self, values):
        min, max = values
        self._context.set_x_view_limits(SliceDirection.crossline, min, max)
        self._context.set_x_view_limits(SliceDirection.depth, min, max)

    def xline_limits_changed(self, values):
        min, max = values
        self._context.set_x_view_limits(SliceDirection.inline, min, max)
        self._context.set_y_view_limits(SliceDirection.depth, min, max)



import os
from unittest import TestCase
from segyviewlib import resource_icon_path


class TestSegyView(TestCase):
    def setUp(self):
        self.filename = "tests/testdata/small.sgy"

    def test_resources(self):
        icons = ['cog.png',
                 'folder.png',
                 'layouts_four_grid.png',
                 'layouts_single.png',
                 'layouts_three_bottom_grid.png',
                 'layouts_three_horizontal_grid.png',
                 'layouts_three_left_grid.png',
                 'layouts_three_right_grid.png',
                 'layouts_three_top_grid.png',
                 'layouts_three_vertical_grid.png',
                 'layouts_two_horizontal_grid.png',
                 'layouts_two_vertical_grid.png',
                 'table_export.png']

        for icon in icons:
            path = resource_icon_path(icon)
            print(path)
            self.assertTrue(os.path.exists(path))



#!/usr/bin/env python

from setuptools import setup

long_description = """
=======
SEGYVIEWER
=======

Segyviewer is a small LGPL licensed python library for easy viewing of SEG-Y
files. It uses the segyio library for reading files.


"""

requires = map(str.rstrip, open('requirements.txt').readlines())

setup(name='segyviewer',
      use_scm_version=True,
      description='Simple viewer for SEG-Y files',
      long_description=long_description,
      author='Statoil ASA',
      author_email='fg_gpl@statoil.com',
      url='https://github.com/Statoil/segyviewer',
      install_requires=['segyviewlib'],
      setup_requires=['setuptools_scm'],
      scripts=['applications/segyviewer'],
      license='LGPL-3.0',
      platforms='any',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Other Environment',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
          'Natural Language :: English',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Topic :: Scientific/Engineering',
          'Topic :: Scientific/Engineering :: Physics',
          'Topic :: Software Development :: Libraries',
          'Topic :: Utilities'
      ]
      )

setup(name='segyviewlib',
      use_scm_version={'write_to': 'src/segyviewlib/version.py'},
      description='Simple viewer library for SEG-Y files',
      long_description=long_description,
      author='Statoil ASA',
      author_email='fg_gpl@statoil.com',
      url='https://github.com/Statoil/segyviewer',
      install_requires=requires,
      setup_requires=['setuptools_scm'],
      packages=['segyviewlib'],
      test_suite='tests',
      package_dir={'': 'src'},
      package_data={'':
                        ['LICENSE',
                         'README.md',
                         'resources/img/350px-SEGYIO.png',
                         'resources/img/cog.png',
                         'resources/img/layouts_four_grid.png',
                         'resources/img/layouts_three_bottom_grid.png',
                         'resources/img/layouts_three_left_grid.png',
                         'resources/img/layouts_three_top_grid.png',
                         'resources/img/layouts_two_horizontal_grid.png',
                         'resources/img/folder.png',
                         'resources/img/layouts_single.png',
                         'resources/img/layouts_three_horizontal_grid.png',
                         'resources/img/layouts_three_right_grid.png',
                         'resources/img/layouts_three_vertical_grid.png',
                         'resources/img/layouts_two_vertical_grid.png',
                         'resources/img/table_export.png',
                         'resources/img/readme.txt'
                         ]},
      license='LGPL-3.0',
      platforms='any',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Other Environment',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
          'Natural Language :: English',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Topic :: Scientific/Engineering',
          'Topic :: Scientific/Engineering :: Physics',
          'Topic :: Software Development :: Libraries',
          'Topic :: Utilities'
      ]
      )
